#!/open_internet/arch/i386-unknown-freebsd2.1/bin/tcl
#
#  This version of NeoWebScript changes the database format.  After you
#  have built the new version, but before you run it, you must convert
#  any .db files under the neoscript-data directory (except the system
#  URL counter database -- it's interface did not change).

#  This script will:
#  1. terminate the running httpd script.
#  2. run through neoscript-data/users and convert the .db files.
#  3. go to the 'conf' directory and copy in common.tcl.new and init.tcl.new.
#  4. restart the webserver.

#  It must be run as root.  If this is your first version of NeoWebScript,
#  you do not have anything to convert.

#  Before running this script, make sure that you back up your database
#  files, just in case, and review for any pathname changes.
#
#  Aside from the pathnames below, you should not have to change any
#  other code.  If you have the libneo7.5 stuff linked into a Tcl
#  interpreter, then comment out the load command below.

load /open_internet/arch/i386-unknown-freebsd2.1/lib/libneo7.5.so

# 'top' is the directory where the files are that will be converted.
set top /usr/local/etc/httpd

# 'userid' and 'groupid' are the user and group ids of your webserver.
# These are used to setuid to them so that the newly converted files
# will have the correct owner and group.
set userid neobot
set groupid neobot
umask 02

# stop the httpd server process
puts "Stopping HTTPD"
while {![catch {kill [read_file -nonewline $top/logs/httpd.pid]; sleep 1}]} { }

flush stdout

if [fork]==0 {

id group $groupid
id user $userid
puts "Converting database files"
cd  $top/neoscript-data/users
foreach dbdir [glob */*/db] {
    foreach dbfile [glob $dbdir/*.db] {
	if [file exists $dbfile.OLD] continue
	puts "converting $dbfile"
	if [catch {set db [db open $dbfile hash r]}] {
		puts "error openning $dbfile"
		exec mv $dbfile $dbfile.NOPEN
		continue
	}
	set dbw [db open $dbfile.new hash ct 0664]
	if [catch {
		db forall $db key {
			db get $db $key data
			set data [eval concat $data]
			db put $dbw $key $data
		}
	}] {
		puts "$dbfile buggy"
		db close $dbw
		db close $db
		unlink $dbfile.new
		exec mv $dbfile $dbfile.BUGGY
		continue
	}
	db close $dbw
	db close $db
	exec mv $dbfile $dbfile.OLD
	exec mv $dbfile.new $dbfile
    }
}

} {
wait
}

cd $top/conf

puts "About to finish up.  Hit return to continue"
gets stdin

if [file exists init.tcl.new] {
	puts "moving in init.tcl.new"
	exec cp init.tcl init.tcl.old
	exec mv init.tcl.new init.tcl
}

if [file exists common.tcl.new] {
	puts "moving in common.tcl.new"
	exec cp common.tcl common.tcl.old
	exec mv common.tcl.new common.tcl
}

puts "Restarting webserver"

exec /usr/devel/httpd/httpd -d $top

puts "Done converting web site"
